@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 11751 2019-10-16 02:58:57Z Lavender $
+# $Id: __init__.py 11780 2019-11-19 10:53:34Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $ (last)
-# $Date: 2019-10-16 11:58:57 +0900 (週三, 16 十月 2019) $
-# $Revision: 11751 $
+# $Date: 2019-11-19 19:53:34 +0900 (週二, 19 十一月 2019) $
+# $Revision: 11780 $
 
 import os
 import sys
@@ -423,6 +423,16 @@ def attachSettings(settingsDict, enablePaypal=False):
         else:
             settingsDict['OSCAR_DASHBOARD_NAVIGATION'] = \
                                             ADDING_OSCAR_DASHBOARD_NAVIGATION
+                                            
+    # bonus
+    for s in settingsDict['OSCAR_DASHBOARD_NAVIGATION']:
+        if s.get('icon') == 'icon-bullhorn': # icon
+            s['children'].append({
+                'label': _('Bonus settings'),
+                'url_name': 'bonusSettings',
+                'access_fn': \
+                    lambda user, url_name, url_args, url_kwargs: user.is_staff,
+            })
     #=========================================================================
     
     #=========================================================================
@@ -542,6 +552,11 @@ def attachShopURLs(settings, urlpatterns):
         url(r'^i18n/', include('django.conf.urls.i18n')),
         url(r'^checkout/payment/redirect/', 
             views.sendPaymentForm, name="formRedirect"),
+        url(r'^shop/dashboard/bonus/', 
+            staff_member_required(views.BonusSettingsView.as_view()), 
+            name="bonusSettings"),
+        url(r'^shop/basket/bonus',
+            views.useBonus, name="useBonus"),
     )
     if VERSION >= (2, 0, 0):
         from django.apps import apps
