@@ -121,6 +121,57 @@ def isOrdererAddress(self):
         
 UserAddress.isOrdererAddress = isOrdererAddress
 
+class Bonus(models.Model):
+    user = models.ForeignKey(
+        User,
+        verbose_name=_("User"),
+    )
+    
+    quantity = models.PositiveIntegerField(
+        verbose_name=_("Quantity"), default=0)
+        
+@property
+def bonus(self):
+    b, created = Bonus.objects.get_or_create(user=self)
+    return b.quantity
+    
+User.bonus = bonus
+    
+class BonusRecord(models.Model):
+    user = models.ForeignKey(
+        User,
+        verbose_name=_("User"),
+    )
+    order = models.ForeignKey(
+        Order,
+        verbose_name=_("Order"),
+    )
+    
+    
+    use = models.PositiveIntegerField(
+        verbose_name=_("Use"), default=0)
+    receive = models.PositiveIntegerField(
+        verbose_name=_("Receive"), default=0)
+        
+    createdTime = models.DateTimeField(_("Created Time"), auto_now_add=True)
+    
+@property
+def bonusRecords(self):
+    return BonusRecord.objects.filter(user=self)
+    
+User.bonusRecords = bonusRecords    
+    
+class BonusSettings(models.Model):
+    consumption = models.PositiveIntegerField(
+        verbose_name=_("Consumption exceed"), default=0)
+    percent = models.PositiveIntegerField(
+        verbose_name=_("Percent of consumption turn into bouns"), default=0)
+    discount = models.PositiveIntegerField(
+        verbose_name=_("Discount per bonus"), default=0)
+    limit = models.PositiveIntegerField(
+        verbose_name=_("Dividend use cap"), default=0)
+    
+    
 class ECPayTrade(models.Model):
     merchantId = models.CharField(max_length=255)
     merchantTradeNo = models.CharField(max_length=255)
