@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: views.py 11774 2019-11-05 06:26:48Z Lavender $
+# $Id: views.py 11780 2019-11-19 10:53:34Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,12 +12,13 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2019-11-05 15:26:48 +0900 (週二, 05 十一月 2019) $
-# $Revision: 11774 $ 
+# $Date: 2019-11-19 19:53:34 +0900 (週二, 19 十一月 2019) $
+# $Revision: 11780 $ 
 
 import json
 import logging
 import hashlib
+import datetime
 import base64
 import urllib
 import six
@@ -55,7 +56,8 @@ from Iuno.shop import forms
 from Iuno.shop import buildAPIForm
 from Iuno.shop.models import (
     ECPayTrade, AllPayTrade, SpgatewayTrade, CathaybkCreditTrade,
-    ECPayLogisticTrade, OrderOrdererAddress, OrdererAddress)
+    ECPayLogisticTrade, OrderOrdererAddress, OrdererAddress, 
+    BonusSettings, BonusRecord, Bonus)
 from Iuno.shop.Util import createCheckValue
 
 _PaymentDetailsView = get_class('checkout.views', 'PaymentDetailsView')
@@ -264,6 +266,42 @@ def placeOrder(self, basket, total,  # noqa (too complex (12))
                         'address': address,
                     }
                 )
+        
+        # bonus        
+        if user.is_authenticated:
+            b = Bonus.objects.get(user=user)
+            config = BonusSettings.objects.get(id=1)
+            
+            # receive
+            consumption = int(total.excl_tax) - config.consumption 
+            receive = 0
+            if consumption >= 0:
+                # config.percent / 100 = X %
+                receive = int(consumption * (config.percent / 100))
+                
+            # use
+            use = 0
+            bonusVoucher = None
+            for v in basket.vouchers.all():
+                if v.code.startswith("BONUS"):
+                    bonusVoucher = v
+              
+            # len(BONUS-[bouns use]-[bouns discount]-XXXX.split("-")) == 4
+            if bonusVoucher:
+                if len(bonusVoucher.code.split("-")) == 4: 
+                    use = int(bonusVoucher.code.split("-")[1])
+                    
+            # update
+            b.quantity = b.quantity + receive - use
+            b.save()
+            
+            # record
+            record = BonusRecord.objects.create(
+                user=user,
+                order=order,
+                use=use,
+                receive=receive,
+            )
             
     return order
     
@@ -418,6 +456,163 @@ def handle_order_status_change(self, order, new_status, note_msg=None):
 
 EventHandler.handle_order_status_change = handle_order_status_change
 
+# https://code.nuwainfo.com/trac/Mercurius/ticket/3285#ticket
+from oscar.apps.offer import conditions
+
+def is_satisfied(self, offer, basket):
+    """
+    Determines whether a given basket meets this condition
+    """
+    num_matches = 0
+    for line in basket.all_lines():
+        if self.can_apply_condition(line):
+            num_matches += line.quantity_without_offer_discount(offer)
+        if num_matches >= self.value:
+            return True
+    return False 
+    
+conditions.CountCondition.is_satisfied = is_satisfied
+
+class BonusSettingsView(generic.TemplateView):
+    
+    template_name = 'shop/offer/dashboard/Bonus.html'
+    
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)    
+        config, created = BonusSettings.objects.get_or_create(id=1)        
+        context['form'] = forms.BonusSettingsForm(instance=config)
+        return context
+
+    def post(self, request, *args, **kwargs):
+        config, created = BonusSettings.objects.get_or_create(id=1)  
+        form = forms.BonusSettingsForm(request.POST, instance=config)
+        form.save()
+        messages.success(request, _("Settings saved"))
+        return redirect("bonusSettings")
+        
+from oscar.apps.basket import views as basketViews
+
+originBasketViewGetContextData =  basketViews.BasketView.get_context_data
+
+def basketViewGetContextData(self, **kwargs):
+    context = originBasketViewGetContextData(self, **kwargs)
+    context['bonusForm'] = forms.BonusForm()
+    return context
+    
+basketViews.BasketView.get_context_data = basketViewGetContextData
+
+originVoucherRemoveViewPost = basketViews.VoucherRemoveView.post
+
+def voucherRemoveViewPost(self, request, *args, **kwargs):
+    response = redirect('basket:summary')
+
+    voucher_id = kwargs['pk']
+    if not request.basket.id:
+        # Hacking attempt - the basket must be saved for it to have
+        # a voucher in it.
+        return response
+    try:
+        voucher = request.basket.vouchers.get(id=voucher_id)
+    except ObjectDoesNotExist:
+        messages.error(
+            request, _("No voucher found with id '%s'") % voucher_id)
+    else:
+        request.basket.vouchers.remove(voucher)
+        self.remove_signal.send(
+            sender=self, basket=request.basket, voucher=voucher)
+        if voucher.code.startswith("BONUS"):
+            messages.info(
+                request, _("Bouns discount removed from basket"))
+        else:
+            messages.info(
+                request, _("Voucher '%s' removed from basket") % voucher.code)
+
+    return response
+    
+basketViews.VoucherRemoveView.post = voucherRemoveViewPost
+
+Voucher = get_model('voucher', 'Voucher')
+ConditionalOffer = get_model('offer', 'ConditionalOffer')
+Benefit = get_model('offer', 'Benefit')
+Range = get_model('offer', 'Range')
+Condition = get_model('offer', 'Condition')
+
+def useBonus(request):
+    quantity = int(request.POST.get('quantity', 0))
+    
+    if quantity <= 0:
+        messages.error(request, _("Use bonus must exceed 0."))
+        return redirect("basket:summary")
+    b, created = Bonus.objects.get_or_create(user=request.user)
+    config, created = BonusSettings.objects.get_or_create(id=1)
+    
+    if quantity >= b.quantity:
+        messages.error(request, _("Not enough bonus."))
+        return redirect("basket:summary")
+    if quantity >= config.limit:
+        messages.error(
+            request, _("Use bonus over the upper limit %s.") % config.limit)
+        return redirect("basket:summary")
+        
+    value = quantity * config.discount
+    
+    # create voucher
+    range, created = Range.objects.get_or_create(
+        name=_("All Products"),
+        includes_all_products=True,
+    )
+    if Condition.objects.filter(
+        range=range, type=Condition.COUNT, value=1).exists():
+        condition = Condition.objects.filter(
+            range=range, type=Condition.COUNT, value=1).first()
+    else:
+        condition, created = Condition.objects.get_or_create(
+            range=range,
+            type=Condition.COUNT,
+            value=1, # 商品至少有一個
+        )
+
+    if Benefit.objects.filter(
+        range=range, type=Benefit.FIXED, value=value).exists():
+        benefit = Benefit.objects.filter(
+            range=range, type=Benefit.FIXED, value=value).first()
+    else:
+        benefit, created = Benefit.objects.get_or_create(
+            range=range,
+            type=Benefit.FIXED,
+            value=value
+        )
+    
+    name = _("Bouns discount %(value)d for %(username)s") % {
+        'value': value, 'username': request.user.username}
+    offer, created = ConditionalOffer.objects.get_or_create(
+        name=_("Offer for voucher '%s'") % name,
+        offer_type=ConditionalOffer.VOUCHER,
+        benefit=benefit,
+        condition=condition,
+        exclusive=False, # False: 可與其他優惠同時使用
+    )
+    code = "BONUS-%d-%d-%s" % (quantity, value, hashlib.md5(
+        (name + settings.SECRET_KEY).encode()).hexdigest())
+    voucher, created = Voucher.objects.get_or_create(
+        name=name,
+        code=code.upper(),
+        usage=Voucher.MULTI_USE,
+        defaults={
+            'start_datetime': datetime.datetime(1999, 1, 1), # FIXME
+            'end_datetime': datetime.datetime(2099, 12, 31), # FIXME
+        }
+    )
+    voucher.offers.add(offer)
+    
+    # add voucher
+    for v in request.basket.vouchers.all():
+        if v.code.startswith("BONUS"):
+            request.basket.vouchers.remove(v)
+    request.basket.vouchers.add(voucher)
+        
+    return redirect("basket:summary")
+
 # ------------------------------------------------------------------------------
 # /cathaybk/preview/
 # /cathaybk-credit/checkxml/
